<?php
if (!defined('ABSPATH')) exit;

/**
 * JSON数据管理类
 * 负责处理所有与JSON缓存相关的操作
 */
class JefRSSFriend_JSON_Manager {
    
    private static $instance = null;
    private $json_file;
    private $lock_file;
    
    /**
     * 获取单例实例
     */
    public static function get_instance() {
        if (null === self::$instance) {
            self::$instance = new self();
        }
        return self::$instance;
    }
    
    /**
     * 构造函数
     */
    private function __construct() {
        $this->json_file = JEFRSSFRIEND_PATH . 'data/latest.json';
        $this->lock_file = JEFRSSFRIEND_PATH . 'data/json.lock';
    }
    
    /**
     * 更新JSON缓存的统一入口
     * 
     * @return bool 更新是否成功
     */
    public function update_json_cache() {
        try {
            // 检查更新锁
            if (!$this->acquire_lock()) {
                error_log('JSON更新正在进行中，跳过本次更新');
                return false;
            }
            
            // 记录开始更新
            JefRSSFriend_Logger::log_manual('start', null, null, 'info', 'JSON缓存更新开始');
            
            // 生成JSON数据
            $json_data = $this->generate_json_data();
            if (empty($json_data)) {
                throw new Exception('生成JSON数据失败');
            }
            
            // 确保目录存在
            $this->ensure_directory_exists();
            
            // 写入临时文件
            $temp_file = $this->json_file . '.tmp';
            if (file_put_contents($temp_file, $json_data) === false) {
                throw new Exception('写入临时JSON文件失败');
            }
            
            // 原子性替换文件
            if (!rename($temp_file, $this->json_file)) {
                unlink($temp_file);
                throw new Exception('替换JSON文件失败');
            }
            
            // 更新最后更新时间
            update_option('jefrssfriend_last_json_update', current_time('mysql'));
            
            // 记录成功日志
            JefRSSFriend_Logger::log_manual('complete', null, null, 'success', 'JSON缓存更新成功');
            
            return true;
            
        } catch (Exception $e) {
            error_log('JSON更新失败: ' . $e->getMessage());
            JefRSSFriend_Logger::log_manual('error', null, null, 'error', 
                'JSON缓存更新失败: ' . $e->getMessage());
            return false;
            
        } finally {
            // 释放锁
            $this->release_lock();
        }
    }
    
    /**
     * 生成JSON数据
     * 
     * @return string|false
     */
    private function generate_json_data() {
        global $wpdb;
        
        try {
            // 获取设置
            $options = get_option('jefrssfriend_options');
            $latest_count = isset($options['latest_count']) ? intval($options['latest_count']) : 100;
            
            // 获取最新文章
            $articles = $wpdb->get_results($wpdb->prepare("
                SELECT 
                    a.*, 
                    s.title as site_title,
                    s.site_url,
                    s.icon_url,
                    g.name as group_name
                FROM {$wpdb->prefix}jefrss_articles a
                LEFT JOIN {$wpdb->prefix}jefrss_sites s ON a.site_id = s.id
                LEFT JOIN {$wpdb->prefix}jefrss_groups g ON s.group_id = g.id
                WHERE s.status = 'active'
                ORDER BY a.pub_date DESC
                LIMIT %d
            ", $latest_count));
            
            if (empty($articles)) {
                return json_encode(array());
            }
            
            // 处理文章数据
            $processed_articles = array();
            foreach ($articles as $article) {
                $processed_articles[] = array(
                    'id' => $article->id,
                    'title' => $article->title,
                    'link' => $article->link,
                    'description' => $this->get_excerpt($article->content),
                    'pub_date' => $article->pub_date,
                    'site' => array(
                        'id' => $article->site_id,
                        'title' => $article->site_title,
                        'url' => $article->site_url,
                        'icon' => $article->icon_url,
                        'group' => $article->group_name
                    )
                );
            }
            
            return json_encode($processed_articles, JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT);
            
        } catch (Exception $e) {
            error_log('生成JSON数据失败: ' . $e->getMessage());
            return false;
        }
    }
    
    /**
     * 获取文章摘要
     */
    private function get_excerpt($content, $length = 150) {
        $excerpt = strip_tags($content);
        $excerpt = trim($excerpt);
        if (mb_strlen($excerpt) > $length) {
            $excerpt = mb_substr($excerpt, 0, $length) . '...';
        }
        return $excerpt;
    }
    
    /**
     * 确保目录存在
     */
    private function ensure_directory_exists() {
        $dir = dirname($this->json_file);
        if (!file_exists($dir)) {
            if (!mkdir($dir, 0755, true)) {
                throw new Exception('创建目录失败: ' . $dir);
            }
        }
    }
    
    /**
     * 获取锁
     */
    private function acquire_lock() {
        if (file_exists($this->lock_file)) {
            $lock_time = filemtime($this->lock_file);
            // 如果锁超过5分钟，认为是死锁
            if (time() - $lock_time < 300) {
                return false;
            }
            unlink($this->lock_file);
        }
        
        return touch($this->lock_file);
    }
    
    /**
     * 释放锁
     */
    private function release_lock() {
        if (file_exists($this->lock_file)) {
            unlink($this->lock_file);
        }
    }
    
    /**
     * 检查是否需要更新
     * 
     * @return bool
     */
    public function needs_update() {
        // 如果文件不存在，需要更新
        if (!file_exists($this->json_file)) {
            return true;
        }
        
        // 获取上次更新时间
        $last_update = get_option('jefrssfriend_last_json_update');
        if (!$last_update) {
            return true;
        }
        
        // 获取更新间隔设置
        $options = get_option('jefrssfriend_options');
        $interval = isset($options['json_refresh_interval']) ? 
                   intval($options['json_refresh_interval']) : 60; // 默认60分钟
        
        // 检查是否超过更新间隔
        $time_diff = time() - strtotime($last_update);
        return $time_diff >= ($interval * 60);
    }
    
    /**
     * 获取JSON数据
     * 
     * @return array|false
     */
    public function get_json_data() {
        if (!file_exists($this->json_file)) {
            return false;
        }
        
        $json_content = file_get_contents($this->json_file);
        if ($json_content === false) {
            return false;
        }
        
        $data = json_decode($json_content, true);
        if (json_last_error() !== JSON_ERROR_NONE) {
            error_log('JSON解析错误: ' . json_last_error_msg());
            return false;
        }
        
        return $data;
    }
} 