<?php
if (!defined('ABSPATH')) exit;

class JefRSSFriend_Logger {
    private static function write($type, $action, $site_id = null, $site_url = null, $status = 'success', $message = '') {
        global $wpdb;
        
        // 直接使用服务器时间并根据WordPress时区设置转换
        $utc_timestamp = time(); // 获取 UTC 时间戳
        $timezone = get_option('timezone_string');
        
        if (empty($timezone)) {
            // 如果没有设置时区,使用偏移量
            $gmt_offset = get_option('gmt_offset');
            $timezone = timezone_name_from_abbr('', $gmt_offset * 3600, 0);
        }
        
        try {
            $date = new DateTime('@' . $utc_timestamp);
            $date->setTimezone(new DateTimeZone($timezone));
            $local_time = $date->format('Y-m-d H:i:s');
        } catch (Exception $e) {
            // 如果时区转换失败,回退到 WordPress 默认方法
            $local_time = current_time('mysql');
            error_log('Time conversion failed: ' . $e->getMessage());
        }
        
        // 如果有站点ID但没有URL，尝试从数据库获取
        if ($site_id && !$site_url) {
            $site_url = $wpdb->get_var($wpdb->prepare(
                "SELECT site_url FROM {$wpdb->prefix}jefrss_sites WHERE id = %d",
                $site_id
            ));
        }
        
        return $wpdb->insert(
            $wpdb->prefix . 'jefrss_logs',
            array(
                'type' => $type,
                'action' => $action,
                'site_id' => $site_id,
                'site_url' => $site_url,
                'status' => $status,
                'message' => $message,
                'created_at' => $local_time
            )
        );
    }
    
    // 记录定时任务日志
    public static function log_cron($action, $site_id = null, $site_url = null, $status = 'success', $message = '') {
        return self::write('cron', $action, $site_id, $site_url, $status, $message);
    }
    
    // 记录手动操作日志
    public static function log_manual($action, $site_id = null, $site_url = null, $status = 'success', $message = '') {
        return self::write('manual', $action, $site_id, $site_url, $status, $message);
    }
    
    /**
     * 清理日志
     * @param string|int $days 要清理的天数，'all' 表示清理全部
     * @return array 清理结果
     */
    public static function cleanup_logs($days) {
        global $wpdb;
        
        try {
            if ($days === 'all') {
                // 清理全部日志
                $result = $wpdb->query("TRUNCATE TABLE {$wpdb->prefix}jefrss_logs");
                $affected = $wpdb->get_var("SELECT ROW_COUNT()");
                
                return array(
                    'success' => true,
                    'message' => sprintf(__('已清理全部日志，共 %d 条', 'jefrssfriend'), $affected)
                );
            } else {
                // 清理指定天数前的日志
                $days = intval($days);
                $cutoff_date = date('Y-m-d H:i:s', strtotime("-{$days} days"));
                
                $result = $wpdb->query($wpdb->prepare(
                    "DELETE FROM {$wpdb->prefix}jefrss_logs WHERE created_at < %s",
                    $cutoff_date
                ));
                
                return array(
                    'success' => true,
                    'message' => sprintf(__('已清理 %d 天前的日志，共 %d 条', 'jefrssfriend'), $days, $result)
                );
            }
        } catch (Exception $e) {
            return array(
                'success' => false,
                'message' => sprintf(__('清理日志失败：%s', 'jefrssfriend'), $e->getMessage())
            );
        }
    }
    
    /**
     * 处理清理日志的AJAX请求
     */
    public static function handle_cleanup_request() {
        // 检查权限
        if (!current_user_can('manage_options')) {
            wp_send_json_error(array('message' => __('权限不足', 'jefrssfriend')));
        }
        
        // 验证nonce
        if (!check_ajax_referer('jefrssfriend_cleanup_logs', 'cleanup_logs_nonce', false)) {
            wp_send_json_error(array('message' => __('安全验证失败', 'jefrssfriend')));
        }
        
        $days = isset($_POST['cleanup_days']) ? $_POST['cleanup_days'] : '7';
        $result = self::cleanup_logs($days);
        
        if ($result['success']) {
            wp_send_json_success(array(
                'message' => $result['message'],
                'reload' => true
            ));
        } else {
            wp_send_json_error(array('message' => $result['message']));
        }
    }
}

// 添加AJAX处理钩子
add_action('wp_ajax_cleanup_logs', array('JefRSSFriend_Logger', 'handle_cleanup_request')); 