<?php
if (!defined('ABSPATH')) exit;

class JefRSSFriend_Lite_Updater {
    private $api_url = 'https://www.jeffer.xyz/api/updates/jefrssfriend';
    private $plugin_slug;
    private $plugin_base;
    private $version;
    private $cache_key;
    private $cache_allowed;

    public function __construct() {
        $this->plugin_slug = dirname(plugin_basename(JEFRSSFRIEND_PATH . 'jefrssfriend.php'));
        $this->plugin_base = plugin_basename(JEFRSSFRIEND_PATH . 'jefrssfriend.php');
        $this->version = JEFRSSFRIEND_VERSION;
        $this->cache_key = 'jefrssfriend_update_' . $this->version;
        $this->cache_allowed = true;

        add_filter('plugins_api', array($this, 'info'), 20, 3);
        add_filter('site_transient_update_plugins', array($this, 'update'));
        add_action('upgrader_process_complete', array($this, 'purge'), 10, 2);

        // 注册定时任务钩子
        add_action('jefrssfriend_cron_refresh', array($this, 'refresh_feeds'));
        
        // 获取设置的刷新间隔
        $options = get_option('jefrssfriend_options');
        $refresh_interval = isset($options['refresh_interval']) ? intval($options['refresh_interval']) : 60; // 默认60分钟
        
        // 如果间隔有变化，重新设置定时任务
        $current_interval = wp_get_schedule('jefrssfriend_cron_refresh');
        $desired_interval = $this->get_cron_interval($refresh_interval);
        
        if ($current_interval !== $desired_interval) {
            // 清除现有的定时任务
            wp_clear_scheduled_hook('jefrssfriend_cron_refresh');
            
            // 重新设置定时任务
            if (!wp_next_scheduled('jefrssfriend_cron_refresh')) {
                wp_schedule_event(time(), $desired_interval, 'jefrssfriend_cron_refresh');
            }
        }
    }

    public function request() {
        $remote = get_transient($this->cache_key);

        if (false === $remote || !$this->cache_allowed) {
            $remote = wp_remote_get(
                $this->api_url,
                array(
                    'timeout' => 10,
                    'headers' => array(
                        'Accept' => 'application/json'
                    )
                )
            );

            if (
                is_wp_error($remote) || 
                200 !== wp_remote_retrieve_response_code($remote) || 
                empty(wp_remote_retrieve_body($remote))
            ) {
                return false;
            }

            set_transient($this->cache_key, $remote, DAY_IN_SECONDS);
        }

        $remote = json_decode(wp_remote_retrieve_body($remote));

        return $remote;
    }

    public function info($res, $action, $args) {
        // 只在查看插件详情时处理
        if ('plugin_information' !== $action) {
            return $res;
        }

        // 确保是查询我们的插件
        if ($this->plugin_slug !== $args->slug) {
            return $res;
        }

        $remote = $this->request();

        if (!$remote) {
            return $res;
        }

        $res = new stdClass();

        $res->name = $remote->name;
        $res->slug = $remote->slug;
        $res->version = $remote->version;
        $res->tested = $remote->tested;
        $res->requires = $remote->requires;
        $res->author = $remote->author;
        $res->author_profile = $remote->author_profile;
        $res->download_link = $remote->download_url;
        $res->trunk = $remote->download_url;
        $res->requires_php = $remote->requires_php;
        $res->last_updated = $remote->last_updated;

        $res->sections = array(
            'description' => $remote->sections->description,
            'installation' => $remote->sections->installation,
            'changelog' => $remote->sections->changelog
        );

        if (!empty($remote->banners)) {
            $res->banners = array(
                'low' => $remote->banners->low,
                'high' => $remote->banners->high
            );
        }

        return $res;
    }

    public function update($transient) {
        if (empty($transient->checked)) {
            return $transient;
        }

        $remote = $this->request();

        if (
            $remote && 
            version_compare($this->version, $remote->version, '<') && 
            version_compare($remote->requires, get_bloginfo('version'), '<=') && 
            version_compare($remote->requires_php, PHP_VERSION, '<')
        ) {
            $res = new stdClass();
            $res->slug = $this->plugin_slug;
            $res->plugin = $this->plugin_base;
            $res->new_version = $remote->version;
            $res->tested = $remote->tested;
            $res->package = $remote->download_url;

            $transient->response[$res->plugin] = $res;
        }

        return $transient;
    }

    public function purge($upgrader, $options) {
        if (
            $this->cache_allowed && 
            'update' === $options['action'] && 
            'plugin' === $options['type']
        ) {
            // 只有更新插件时才清除缓存
            delete_transient($this->cache_key);
        }
    }

    /**
     * 处理文章内容优化
     */
    public function process_article_content($content) {
        if (empty($content)) {
            return array(
                'content' => '',
                'images' => array()
            );
        }

        try {
            // DEBUG: 记录处理前的内容
            error_log('DEBUG: process_article_content 输入内容类型: ' . gettype($content));
            error_log('DEBUG: process_article_content 输入内容前100字符: ' . substr($content, 0, 100));
            
            // 处理图片
            $processed_content = $this->process_images($content);
            
            // DEBUG: 记录处理后的内容
            error_log('DEBUG: process_article_content 输出内容类型: ' . gettype($processed_content));
            error_log('DEBUG: process_article_content 输出内容前100字符: ' . substr($processed_content, 0, 100));
            
            return array(
                'content' => $processed_content,
                'images' => array()
            );
        } catch (Exception $e) {
            error_log('Content Processing Error: ' . $e->getMessage());
            return array(
                'content' => $content,
                'images' => array()
            );
        }
    }
    
    /**
     * 验证HTML内容
     */
    private function validate_html($content) {
        // 基本的HTML验证
        if (substr_count($content, '<') !== substr_count($content, '>')) {
            return false;
        }
        
        // 检查是否有明显的HTML错误
        $invalid_patterns = array(
            '="[^"]*"=""',  // 检测重复的空属性
            '<[^>]*[^a-zA-Z0-9-_]=[^>]*>', // 检测无效的属性格式
        );
        
        foreach ($invalid_patterns as $pattern) {
            if (preg_match('/' . $pattern . '/', $content)) {
                return false;
            }
        }
        
        return true;
    }
    
    /**
     * 处理图片标签
     */
    private function process_images($content) {
        try {
            // 使用 DOMDocument 处理 HTML
            $dom = new DOMDocument();
            libxml_use_internal_errors(true);
            
            // 关键：添加一个包装器来保护HTML结构
            $wrapped_content = '<div>' . $content . '</div>';
            $dom->loadHTML($wrapped_content, LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD | LIBXML_NOERROR);
            
            // 获取所有图片标签
            $images = $dom->getElementsByTagName('img');
            foreach ($images as $img) {
                if (!$img->hasAttribute('loading')) {
                    $img->setAttribute('loading', 'lazy');
                }
            }
            
            // 获取处理后的HTML
            $processed_content = $dom->saveHTML($dom->documentElement);
            
            // 移除包装器div
            $processed_content = preg_replace('/^<div>|<\/div>$/', '', $processed_content);
            
            // DEBUG: 验证HTML结构
            error_log('DEBUG: HTML结构验证 - 开始标签数: ' . substr_count($processed_content, '<'));
            error_log('DEBUG: HTML结构验证 - 结束标签数: ' . substr_count($processed_content, '>'));
            
            return $processed_content;
        } catch (Exception $e) {
            error_log('Image processing error: ' . $e->getMessage());
            return $content;
        }
    }

    /**
     * 检测代码语言
     */
    private function detect_code_language($code) {
        // 基础语言检测
        if (strpos($code, '<?php') !== false || strpos($code, '<?=') !== false) {
            return 'php';
        } elseif (strpos($code, 'function') !== false || strpos($code, 'var') !== false) {
            return 'javascript';
        } elseif (strpos($code, '{') !== false && strpos($code, '}') !== false) {
            return 'css';
        }
        return 'plaintext';
    }

    /**
     * 增强代码块处理
     */
    public function enhance_code_blocks($content) {
        try {
            // 查找所有代码块
            preg_match_all('/<pre><code.*?>(.*?)<\/code><\/pre>/is', $content, $matches);
            
            foreach ($matches[0] as $index => $code_block) {
                // 1. 添加语言检测
                $language = $this->detect_code_language($matches[1][$index]);
                
                // 2. 添加代码高亮
                $new_block = str_replace('<code', '<code class="language-' . $language . '"', $code_block);
                
                // 3. 添加复制按钮
                $new_block = '<div class="code-wrapper">' . 
                            '<button class="copy-code">复制代码</button>' . 
                            $new_block . 
                            '</div>';
                
                $content = str_replace($code_block, $new_block, $content);
            }
            
            return $content;
        } catch (Exception $e) {
            error_log('Code block processing error: ' . $e->getMessage());
            return $content;
        }
    }

    /**
     * 刷新所有RSS源
     */
    public function refresh_feeds() {
        try {
            error_log('开始刷新RSS源...');
            
            global $wpdb;
            $sites = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}jefrss_sites WHERE status = 'active'");
            
            foreach ($sites as $site) {
                $rss = fetch_feed($site->feed_url);
                foreach ($rss->get_items() as $item) {
                    $content = $item->get_content();
                    $this->save_article($article_data);
                }
            }
            
            // 生成JSON缓存
            error_log('开始生成JSON缓存...');
            if (function_exists('jefrssfriend_generate_latest_json')) {
                $json_result = jefrssfriend_generate_latest_json();
                error_log('latest.json生成结果: ' . ($json_result ? '成功' : '失败'));
            } else {
                error_log('错误: jefrssfriend_generate_latest_json 函数未定义');
            }
            
            return true;
            
        } catch (Exception $e) {
            error_log('RSS刷新错误: ' . $e->getMessage());
            return false;
        }
    }
    
    /**
     * 保存RSS文章
     */
    private function save_rss_article($site_id, $item) {
        global $wpdb;
        
        // 获取原始内容，不做任何处理
        $content = $item->get_content();
        
        // 准备文章数据
        $article_data = array(
            'site_id' => $site_id,
            'title' => $item->get_title(),
            'link' => $item->get_permalink(),
            'pub_date' => $item->get_date('Y-m-d H:i:s'),
            'content' => $content,  // 直接使用原始内容，不做过滤
            'article_hash' => md5($item->get_permalink()),
            'processed' => 0
        );
        
        // 检查文章是否已存在
        $exists = $wpdb->get_var($wpdb->prepare(
            "SELECT id FROM {$wpdb->prefix}jefrss_articles 
            WHERE site_id = %d AND article_hash = %s",
            $site_id,
            $article_data['article_hash']
        ));
        
        if (!$exists) {
            $result = $wpdb->insert(
                $wpdb->prefix . 'jefrss_articles',
                $article_data,
                array('%d', '%s', '%s', '%s', '%s', '%s', '%d')
            );
            
            if ($result === false) {
                error_log('保存RSS文章失败: ' . $wpdb->last_error);
                return false;
            }
            return $wpdb->insert_id;
        }
        
        return false;
    }

    /**
     * 根据设置的分钟数获取对应的 cron 间隔
     */
    private function get_cron_interval($minutes) {
        add_filter('cron_schedules', function($schedules) use ($minutes) {
            $schedules['jefrssfriend_interval'] = array(
                'interval' => $minutes * 60,
                'display' => sprintf(__('每 %d 分钟', 'jefrssfriend'), $minutes)
            );
            return $schedules;
        });
        return 'jefrssfriend_interval';
    }

    /**
     * 清理重复文章
     */
    private function cleanup_duplicate_articles() {
        global $wpdb;
        
        error_log('开始清理重复文章...');
        
        // 查找并删除重复文章，保留最新的一条
        $sql = "DELETE a1 FROM {$wpdb->prefix}jefrss_articles a1
                INNER JOIN {$wpdb->prefix}jefrss_articles a2
                WHERE a1.site_id = a2.site_id 
                AND a1.article_hash = a2.article_hash
                AND a1.id < a2.id";
                
        $result = $wpdb->query($sql);
        
        if ($result !== false) {
            error_log('清理了 ' . $result . ' 篇重复文章');
        }
    }

    /**
     * 清理所有文章
     */
    public function cleanup_all_articles() {
        global $wpdb;
        
        error_log('开始清理所有文章...');
        
        $result = $wpdb->query("TRUNCATE TABLE {$wpdb->prefix}jefrss_articles");
        
        if ($result !== false) {
            error_log('所有文章已清理完成');
            return true;
        }
        
        error_log('清理文章时发生错误');
        return false;
    }

    public function enhance_image_processing($content) {
        try {
            // 添加图片处理增强
            preg_match_all('/<img[^>]+>/i', $content, $matches);
            
            foreach ($matches[0] as $img_tag) {
                // 1. 添加图片加载状态
                $new_img = str_replace('<img', '<img data-loading="true"', $img_tag);
                
                // 2. 添加错误处理
                $new_img = str_replace('<img', '<img onerror="this.onerror=null;this.classList.add(\'load-failed\')"', $new_img);
                
                // 3. 添加图片预览
                $new_img = '<div class="image-wrapper">' . $new_img . '<div class="image-preview"></div></div>';
                
                $content = str_replace($img_tag, $new_img, $content);
            }
            
            return $content;
        } catch (Exception $e) {
            error_log('Image processing error: ' . $e->getMessage());
            return $content;
        }
    }

    public function update_feed($feed_url) {
        if (empty($feed_url)) {
            return false;
        }

        try {
            $feed = fetch_feed($feed_url);
            if (is_wp_error($feed)) {
                throw new Exception('Failed to fetch feed');
            }

            $items = $feed->get_items();
            foreach ($items as $item) {
                // 获取文章内容
                $content = $item->get_content();
                
                // 使用process_content处理内容,保留HTML
                $processed_content = $this->process_content($content);
                
                $article_data = array(
                    'title' => $item->get_title(),
                    'content' => $processed_content, // 使用处理后的内容
                    'link' => $item->get_permalink(),
                    'keep_html' => true, // 添加HTML保留标记
                    'pub_date' => $item->get_date('Y-m-d H:i:s'),
                    'feed_url' => $feed_url
                );

                // 保存到数据库
                $this->save_article($article_data);
            }

            return true;
        } catch (Exception $e) {
            error_log($e->getMessage());
            return false;
        }
    }

    /**
     * 保存文章到数据库
     */
    private function save_article($article_data) {
        global $wpdb;
        $table = $wpdb->prefix . 'jefrss_articles';
        
        // 检查文章是否已存在
        $exists = $wpdb->get_var($wpdb->prepare(
            "SELECT id FROM $table WHERE link = %s",
            $article_data['link']
        ));

        if (!$exists) {
            // 插入新文章
            $wpdb->insert($table, array(
                'title' => $article_data['title'],
                'content' => $article_data['content'],
                'link' => $article_data['link'], 
                'keep_html' => $article_data['keep_html'],
                'pub_date' => $article_data['pub_date'],
                'feed_url' => $article_data['feed_url']
            ));
        }
    }

    /**
     * 处理文章内容优化
     */
    public function process_content($content) {
        // 保留原始HTML标签,只过滤危险标签
        $allowed_html = array(
            'p' => array(),
            'br' => array(),
            'strong' => array(),
            'em' => array(),
            'h1' => array(),
            'h2' => array(),
            'h3' => array(),
            'ul' => array(),
            'li' => array(),
            'a' => array('href' => array()),
            'img' => array(
                'src' => array(),
                'alt' => array(),
                'title' => array()
            )
        );
        
        // 使用wp_kses保留安全的HTML
        $content = wp_kses($content, $allowed_html);
        
        return $content;
    }

    /**
     * 获取文章时根据标记处理
     */
    public function get_article($id) {
        $article = $this->db->get_row($this->table, array('id' => $id));
        
        if($article && !empty($article->keep_html)) {
            // 保留HTML格式
            return $article->content;
        }
        
        // 默认去除HTML标签
        return wp_strip_all_tags($article->content);
    }
}