<?php
if (!defined('ABSPATH')) exit;

class JefRSSFriend_Lite_Core {
    private $version;
    private $admin;
    private $settings;
    private $public;
    private $updater;
    
    public function __construct() {
        $this->version = JEFRSSFRIEND_VERSION;
        add_filter('cron_schedules', array($this, 'add_cron_interval'));
    }
    
    public function init() {
        $this->admin = new JefRSSFriend_Lite_Admin();
        $this->settings = new JefRSSFriend_Lite_Settings();
        $this->updater = new JefRSSFriend_Lite_Updater();
        
        load_plugin_textdomain('jefrssfriend', false, JEFRSSFRIEND_PATH . 'languages');
        
        add_shortcode('jefrss_latest', array($this, 'render_latest_articles'));
    }
    
    public function activate() {
        error_log('JefRSS: Starting core activation...');
        
        try {
            // 创建必要的目录
            $this->create_directories();
            
            // 创建数据表
            $this->create_tables();
            
            // 添加默认选项
            $this->add_default_options();
            
            // 设置定时任务
            $this->setup_cron_jobs();
            
            // 刷新重写规则
            flush_rewrite_rules();
            
            error_log('JefRSS: Core activation completed');
            
        } catch (Exception $e) {
            error_log('JefRSS activation error: ' . $e->getMessage());
            throw $e;
        }
    }
    
    private function create_tables() {
        global $wpdb;
        
        error_log('JefRSS: Creating database tables...');
        
        try {
            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
            
            $charset_collate = $wpdb->get_charset_collate();
            
            // 添加日志表创建
            $sql_logs = "CREATE TABLE IF NOT EXISTS {$wpdb->prefix}jefrss_logs (
                id bigint(20) NOT NULL AUTO_INCREMENT,
                type varchar(20) NOT NULL COMMENT '日志类型:cron/manual',
                action varchar(50) NOT NULL COMMENT '操作类型:fetch/refresh/error',
                site_id bigint(20) DEFAULT NULL,
                site_url varchar(255) DEFAULT NULL,
                status varchar(20) NOT NULL DEFAULT 'success',
                message text DEFAULT NULL,
                created_at datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
                PRIMARY KEY (id),
                KEY idx_type_created (type, created_at)
            ) $charset_collate;";
            
            dbDelta($sql_logs);
            error_log('JefRSS: Logs table created');
            
            // 分组表
            $sql_groups = "CREATE TABLE IF NOT EXISTS {$wpdb->prefix}jefrss_groups (
                id bigint(20) NOT NULL AUTO_INCREMENT,
                name varchar(255) NOT NULL,
                description text,
                order_num int DEFAULT 0,
                created_at datetime DEFAULT CURRENT_TIMESTAMP,
                PRIMARY KEY (id)
            ) $charset_collate;";
            
            dbDelta($sql_groups);
            error_log('JefRSS: Groups table created');
            
            // 站点表
            $sql_sites = "CREATE TABLE IF NOT EXISTS {$wpdb->prefix}jefrss_sites (
                id bigint(20) NOT NULL AUTO_INCREMENT,
                title varchar(255) NOT NULL,
                rss_url varchar(255) NOT NULL,
                site_url varchar(255) NOT NULL,
                group_id bigint(20) DEFAULT NULL,
                icon_url varchar(255),
                status varchar(20) DEFAULT 'active',
                last_fetch datetime DEFAULT NULL,
                created_at datetime DEFAULT CURRENT_TIMESTAMP,
                updated_at datetime DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
                PRIMARY KEY (id),
                KEY group_id (group_id),
                UNIQUE KEY rss_url (rss_url)
            ) $charset_collate;";
            
            dbDelta($sql_sites);
            error_log('JefRSS: Sites table created');
            
            // 文章表
            $sql_articles = "CREATE TABLE IF NOT EXISTS {$wpdb->prefix}jefrss_articles (
                id bigint(20) NOT NULL AUTO_INCREMENT,
                site_id bigint(20) NOT NULL,
                title varchar(255) NOT NULL,
                link varchar(255) NOT NULL,
                description text,
                content longtext,
                pub_date datetime NOT NULL,
                article_hash varchar(32) NOT NULL,
                created_at datetime DEFAULT CURRENT_TIMESTAMP,
                PRIMARY KEY (id),
                KEY site_id (site_id),
                KEY article_hash (article_hash),
                KEY pub_date (pub_date)
            ) $charset_collate;";
            
            dbDelta($sql_articles);
            error_log('JefRSS: Articles table created');
            
            // 验证表是否创建成功
            $tables = array(
                $wpdb->prefix . 'jefrss_groups',
                $wpdb->prefix . 'jefrss_sites',
                $wpdb->prefix . 'jefrss_articles'
            );
            
            foreach ($tables as $table) {
                if ($wpdb->get_var("SHOW TABLES LIKE '$table'") != $table) {
                    throw new Exception("Failed to create table: $table");
                }
            }
            
            error_log('JefRSS: All tables created successfully');
            return true;
            
        } catch (Exception $e) {
            error_log('JefRSS: Error creating tables: ' . $e->getMessage());
            error_log('JefRSS: MySQL Error: ' . $wpdb->last_error);
            throw $e;
        }
    }
    
    private function create_directories() {
        $dirs = array(
            'data',
            'data/articles',
            'data/cache',
            'data/icons'
        );
        
        foreach ($dirs as $dir) {
            $path = JEFRSSFRIEND_PATH . $dir;
            if (!file_exists($path)) {
                if (!wp_mkdir_p($path)) {
                    throw new Exception(sprintf(
                        '无法创建目录 %s。请检查目录权限。',
                        $path
                    ));
                }
            }
        }
    }
    
    private function add_default_options() {
        $defaults = array(
            'excerpt_length' => 150,
            'fetch_interval' => 'hourly',
            'show_images' => true,
            'latest_json_count' => 100,
            'show_json_count' => 200,
            'cache_time' => 3600,
            'cache_size' => 10,
            'articles_per_feed' => 5
        );
        
        update_option('jefrssfriend_options', $defaults);
        update_option('jefrssfriend_db_version', JEFRSSFRIEND_VERSION);
    }
    
    private function setup_cron_jobs() {
        if (JEFRSSFRIEND_DEBUG) {
            error_log('设置定时任务开始');
        }
        
        // 清理旧的定时任务
        $timestamp = wp_next_scheduled('jefrssfriend_cron_refresh');
        if ($timestamp) {
            wp_unschedule_event($timestamp, 'jefrssfriend_cron_refresh');
            if (JEFRSSFRIEND_DEBUG) {
                error_log('清理旧的定时任务');
            }
        }
        
        // 设置新的定时任务
        $options = get_option('jefrssfriend_options');
        $interval = isset($options['refresh_interval']) ? intval($options['refresh_interval']) : 60;
        
        if (!wp_next_scheduled('jefrssfriend_cron_refresh')) {
            wp_schedule_event(time(), 'jefrssfriend_interval', 'jefrssfriend_cron_refresh');
            if (JEFRSSFRIEND_DEBUG) {
                error_log('新的定时任务已设置，间隔: ' . $interval . ' 分钟');
            }
        }
    }
    
    public function deactivate() {
        // 清理定时任务
        wp_clear_scheduled_hook('jefrssfriend_cron_refresh');
        
        // 清理重写规则
        flush_rewrite_rules();
    }
    
    public function uninstall() {
        global $wpdb;
        
        // 删除数据表
        $tables = array(
            $wpdb->prefix . 'jefrss_sites',
            $wpdb->prefix . 'jefrss_groups',
            $wpdb->prefix . 'jefrss_articles'
        );
        
        foreach ($tables as $table) {
            $wpdb->query("DROP TABLE IF EXISTS $table");
        }
        
        // 删除选项
        delete_option('jefrssfriend_options');
        delete_option('jefrssfriend_db_version');
        
        // 删除文件
        $this->remove_plugin_files();
    }
    
    private function remove_plugin_files() {
        $dirs = array(
            JEFRSSFRIEND_PATH . 'data',
            JEFRSSFRIEND_PATH . 'data/articles',
            JEFRSSFRIEND_PATH . 'data/cache',
            JEFRSSFRIEND_PATH . 'data/icons'
        );
        
        foreach ($dirs as $dir) {
            if (is_dir($dir)) {
                $this->remove_directory($dir);
            }
        }
    }
    
    private function remove_directory($dir) {
        if (is_dir($dir)) {
            $objects = scandir($dir);
            foreach ($objects as $object) {
                if ($object != "." && $object != "..") {
                    if (is_dir($dir . "/" . $object)) {
                        $this->remove_directory($dir . "/" . $object);
                    } else {
                        unlink($dir . "/" . $object);
                    }
                }
            }
            rmdir($dir);
        }
    }
    
    public function upgrade_database() {
        global $wpdb;
        
        $current_version = get_option('jefrssfriend_db_version', '0');
        
        // 如果需要升级
        if (version_compare($current_version, JEFRSSFRIEND_VERSION, '<')) {
            
            // 检查并添加缺失的列
            $table_name = $wpdb->prefix . 'jefrss_articles';
            $columns = $wpdb->get_col("SHOW COLUMNS FROM $table_name");
            
            if (!in_array('site_id', $columns)) {
                $wpdb->query("ALTER TABLE $table_name ADD COLUMN site_id bigint(20) NOT NULL AFTER id");
            }
            
            if (!in_array('description', $columns)) {
                $wpdb->query("ALTER TABLE $table_name ADD COLUMN description text AFTER link");
            }
            
            if (!in_array('article_hash', $columns)) {
                $wpdb->query("ALTER TABLE $table_name ADD COLUMN article_hash varchar(32) NOT NULL AFTER pub_date");
                $wpdb->query("ALTER TABLE $table_name ADD KEY article_hash (article_hash)");
            }
            
            // 更新数据库版本号
            update_option('jefrssfriend_db_version', JEFRSSFRIEND_VERSION);
        }
    }
    
    public function render_latest_articles($atts) {
        try {
            $json_file = JEFRSSFRIEND_PATH . 'data/latest.json';
            error_log('尝试读取JSON文件: ' . $json_file);
            
            if (!file_exists($json_file)) {
                error_log('JSON文件不存在');
                return '<p>暂无文章</p>';
            }
            
            $json_content = file_get_contents($json_file);
            error_log('读取到的JSON内容: ' . substr($json_content, 0, 100) . '...');
            
            $articles = json_decode($json_content, true);
            error_log('JSON解析结果: ' . print_r($articles, true));
            
            if (json_last_error() !== JSON_ERROR_NONE) {
                error_log('JSON解析错误: ' . json_last_error_msg());
                return '<p>数据解析错误</p>';
            }
            
            if (empty($articles)) {
                error_log('文章数组为空');
                return '<p>暂无文章</p>';
            }
            
            error_log('文章数量: ' . count($articles));
            
            $output = '<div class="jefrss-articles">';
            foreach ($articles as $article) {
                error_log('处理文章: ' . $article['title']);
                $output .= $this->render_article_item($article, $atts);
            }
            $output .= '</div>';
            
            return $output;
        } catch (Exception $e) {
            error_log('渲染文章错误: ' . $e->getMessage());
            return '<p>显示文章时出错</p>';
        }
    }
    
    /**
     * 渲染单个文章项目
     */
    private function render_article_item($article, $atts) {
        // 首先验证 $article 是否为数组
        if (!is_array($article)) {
            error_log('JefRSS: Invalid article data format');
            return '';
        }

        try {
            // 使用空值合并运算符设置默认值
            $title = $article['title'] ?? '';
            $link = $article['link'] ?? '';
            $site_title = $article['site_title'] ?? '';
            $pub_date = $article['pub_date'] ?? '';
            $description = $article['description'] ?? '';

            // 验证必要字段
            if (empty($title) || empty($link)) {
                error_log('JefRSS: Missing required article fields');
                return '';
            }

            // 安全地解析URL
            $host = '';
            if (!empty($link)) {
                $parsed_url = parse_url($link);
                $host = $parsed_url['host'] ?? '';
            }
            
            // 修改渲染HTML部分
            $output = '<article style="margin-bottom: 40px; text-align: left !important;">';  // 添加 text-align: left
            $output .= '<header class="entry-header" style="text-align: left !important;">';  // 添加 text-align: left
            $output .= '<div style="
                display: flex; 
                align-items: center; 
                margin-bottom: 15px; 
                width: 100%;
                justify-content: flex-start;
                text-align: left;
            ">';
            
            // 创建一个左侧容器包裹图标和站点名称
            $output .= '<div style="
                display: flex;
                align-items: center;
                flex-shrink: 0;
            ">'; 
            
            // 图标部分
            if (!empty($host)) {
                $output .= '<img src="' . esc_url('https://unavatar.io/' . $host) . '" 
                            style="
                                width: 16px; 
                                height: 16px; 
                                margin-right: 8px; 
                                vertical-align: middle; 
                                object-fit: contain;
                            " 
                            alt="' . esc_attr($site_title) . '"
                            loading="eager"
                            decoding="sync">';
            }
            
            // 站点标题
            $output .= '<span style="
                color: #666; 
                font-size: 14px;
                white-space: nowrap;
            ">' . esc_html($site_title) . '</span>';
            
            // 关闭左侧容器
            $output .= '</div>';
            
            // 日期显示（靠右）
            if (!empty($pub_date)) {
                $timestamp = strtotime($pub_date);
                if ($timestamp !== false) {
                    $output .= '<span style="
                        margin-left: auto; 
                        color: #999; 
                        font-size: 13px;
                        flex-shrink: 0;
                    ">' . human_time_diff($timestamp) . '前</span>';
                }
            }
            
            $output .= '</div>';
            
            // 文章标题
            $output .= '<h3 style="margin: 0; white-space: nowrap; overflow: hidden; text-overflow: ellipsis; text-align: left !important;">';
            $output .= '<a href="' . esc_url($link) . '" target="_blank" style="text-decoration: none; color: #333; text-align: left !important;">';
            $output .= esc_html($title);
            $output .= '</a>';
            $output .= '</h3>';
            $output .= '</header>';
            
            // 文章摘要
            $excerpt_length = isset($atts['excerpt_length']) ? intval($atts['excerpt_length']) : 150;
            if ($excerpt_length > 0 && !empty($description)) {
                $output .= '<div style="margin-top: 15px; text-align: left !important;">';
                $output .= esc_html(wp_trim_words(strip_tags($description), $excerpt_length, '...'));
                $output .= '</div>';
            }
            
            $output .= '<hr style="margin: 30px 0 0 0; border: 0; border-top: 1px solid #eee;">';
            $output .= '</article>';
            
            return $output;
            
        } catch (Exception $e) {
            error_log('JefRSS: Error rendering article: ' . $e->getMessage());
            return '';
        }
    }
    
    public function add_cron_interval($schedules) {
        $options = get_option('jefrssfriend_options');
        $interval = isset($options['refresh_interval']) ? $options['refresh_interval'] : 60;
        
        $schedules['jefrssfriend_interval'] = array(
            'interval' => $interval * 60,  // 转换为秒
            'display' => sprintf(__('每 %d 分钟', 'jefrssfriend'), $interval)
        );
        
        return $schedules;
    }
    
    public function generate_json() {
        global $wpdb;
        
        $articles = $wpdb->get_results("
            SELECT a.*, s.title as site_title, s.site_url 
            FROM {$wpdb->prefix}jefrss_articles a
            LEFT JOIN {$wpdb->prefix}jefrss_sites s ON a.site_id = s.id
            WHERE s.status = 'active'
            ORDER BY a.pub_date DESC
            LIMIT 100
        ");
        
        $data = array();
        foreach ($articles as $article) {
            // 添加字符数限制
            $description = wp_strip_all_tags($article->description ?: $article->content);
            $description = mb_substr($description, 0, 200, 'UTF-8'); // 限制200个字符
            if (mb_strlen($description, 'UTF-8') >= 200) {
                $description .= '...';
            }
            
            $data[] = array(
                'title' => $article->title,
                'link' => $article->link,
                'description' => $description, // 使用处理后的描述
                'pub_date' => $article->pub_date,
                'site_title' => $article->site_title,
                'site_url' => $article->site_url
            );
        }
        
        return json_encode($data);
    }
} 